
;|
;|  Titel....: Line2D
;|  Datei....: Line2D.pbi
;|  Datum....: 09.04.2009
;|  Inhalt...: * Macros und Proceduren fr 2-Dimensionale Geradenberechungen
;|________________________________________________________________________________________________






;  Vorrausgesetzte Includes
;
 
 
 
 XIncludeFile "Vector2D.pbi"






;  Konstanten
;



 #Endless = -1






;  Strukturen
;



 ; Structure einer 2D-Linie
 Structure Line2D
  Position.Vector2D
  Direction.Vector2D
 EndStructure



 ; Structure eines Kreises
 Structure Circle2D
  Position.Vector2D
  Radius.f
 EndStructure






;  Arrays und LinkedLists
;






;  Proceduren und Macros
;
  
  

 ; Setzt die Werte (x,y) und (Width,Height) in die Linie ein 
 Macro SetLine2D(Line2D, xValue, yValue, Width, Height)
  Line2D\Position\x = xValue
  Line2D\Position\y = yValue
  Line2D\Direction\x = Width
  Line2D\Direction\y = Height
 EndMacro



 ; Setzt die Werte (x,y) und (r) in den Kreis ein 
 Macro SetCircle2D(Circle2D, xValue, yValue, RadiusValue)
  Circle2D\Position\x = xValue
  Circle2D\Position\y = yValue
  Circle2D\Radius = RadiusValue
 EndMacro



 ; Setzt die Position in die Linie ein 
 Macro SetPositionLine2D(Line2D, xValue, yValue)
  Line2D\Position\x = xValue
  Line2D\Position\y = yValue
 EndMacro



 ; Setzt die Richtung in die Linie ein 
 Macro SetDirectionLine2D(Line2D, xValue, yValue)
  Line2D\Direction\x = xValue
  Line2D\Direction\y = yValue 
 EndMacro



 ; Gibt die Linie als String aus
 Macro GetLine2D(Line2D)
  "("+Str(Line2D\Position\x)+"|"+Str(Line2D\Position\y)+") --> ("+Str(Line2D\Position\x+Line2D\Direction\x)+"|"+Str(Line2D\Position\y+Line2D\Direction\y)+")"
 EndMacro



 ; berprft zwei Linien auf Kollision und gibt (wenn vorhanden) den Kollisionspunkt aus  
 Procedure CollisionLine2DLine2D(*Line2D_1.Line2D, *Line2D_2.Line2D, *Point2D.Vector2D=0)
  Protected u.f , o.f , i.f , j.f , Vector2D.Vector2D , Temp.f , Norm0.Vector2D , Norm1.Vector2D , Norm2.Vector2D
  u = Vector2DCross(*Line2D_1\Direction, *Line2D_2\Direction)
  If u ; Nicht Parallel
   Vector2DSub(*Line2D_2\Position, *Line2D_1\Position, Vector2D)
   o = Vector2DCross(Vector2D, *Line2D_2\Direction)
   i = (o/u)
   If i >= 0 And i <= 1 ; Kollision auf Line2D_1
    o = Vector2DCross(-Vector2D, *Line2D_1\Direction)
    j = -(o/u)
    If j >= 0 And j <= 1 ; Kollision auf Line2D_2
     If *Point2D
      *Point2D\x = *Line2D_1\Position\x + i * *Line2D_1\Direction\x
      *Point2D\y = *Line2D_1\Position\y + i * *Line2D_1\Direction\y
     EndIf
     ProcedureReturn #True  
    Else ; keine Kollision auf Line2D_2
     ProcedureReturn #False 
    EndIf
   Else ; keine Kollision auf Line2D_1
    ProcedureReturn #False
   EndIf
  Else ; Parallel
   Vector2DSub(*Line2D_2\Position, *Line2D_1\Position, Vector2D)
   If Not Vector2DCross(Vector2D, *Line2D_1\Direction) ; Auf einer Geraden
    TempLength = Vector2DLength(Vector2D)
    Vector2DNorm(Vector2D, Norm0)
    Vector2DNorm(*Line2D_1\Direction, Norm1)
    Vector2DNorm(*Line2D_2\Direction, Norm2)
    If TempLength ; Positionen verschieden
     If CompareVector2D(Norm0, Norm1)
      TempLength - Vector2DLength(*Line2D_1\Direction)
      Move = #True
     EndIf
     RaiseVector2D(Norm0, -1)
     If CompareVector2D(Norm0, Norm2)
      TempLength - Vector2DLength(*Line2D_2\Direction)
     EndIf
    Else ; gleiche Positionen
     If CompareVector2D(Norm1, Norm2)
      ProcedureReturn #Endless
     EndIf
    EndIf 
    If TempLength < 0 ; Unendlich viele Kollisionen
     If *Point2D
      *Point2D\x = *Line2D_1\Position\x
      *Point2D\y = *Line2D_1\Position\y
     EndIf
     ProcedureReturn #Endless
    ElseIf Not TempLength ; Genau eine Kollision
     If *Point2D
      *Point2D\x = *Line2D_1\Position\x + *Line2D_1\Direction\x * Move
      *Point2D\y = *Line2D_1\Position\y + *Line2D_1\Direction\y * Move 
     EndIf
     ProcedureReturn #True
    Else ; keine Kollision
     ProcedureReturn #False
    EndIf
   Else ; Echtparallel
    ProcedureReturn #False
   EndIf
  EndIf  
 EndProcedure
 
 
 
 ; berprft Linie und Kreis auf Kollision und gibt (wenn vorhanden) die Kollisionspunkte aus  
 Procedure CollisionLine2DCircle2D(*Line2D.Line2D, *Circle2D.Circle2D, *Point2D_1.Vector2D=0, *Point2D_2.Vector2D=0)
  Protected Temp.Vector2D, ai.f, bi.f, ci.f, i1.f, i2.f
  Vector2DSub(*Line2D\Position, *Circle2D\Position, Temp)
  ; a*i+b*i+c
  ai = Vector2DScalar(*Line2D\Direction, *Line2D\Direction)
  bi = 2 * Vector2DScalar(Temp, *Line2D\Direction)
  ci = Vector2DScalar(Temp, Temp) - *Circle2D\Radius * *Circle2D\Radius
  VorWurzel.f = - bi / (2*ai)
  Wurzel.f = (bi*bi-4*ai*ci) / (4*ai*ai)
  If Wurzel < 0
   ProcedureReturn #False
  ElseIf Wurzel = 0
   i1.f = VorWurzel
   If i1 >= 0 And i1 <= 1 
    If *Point2D_1
     *Point2D_1\x = *Line2D\Position\x + i1 * *Line2D\Direction\x
     *Point2D_1\y = *Line2D\Position\y + i1 * *Line2D\Direction\y  
    EndIf
    ProcedureReturn #True
   Else
    ProcedureReturn #False
   EndIf
  Else
   i1.f = VorWurzel + Sqr(Wurzel)
   i2.f = VorWurzel - Sqr(Wurzel)
   If i1 >= 0 And i1 <= 1 
    If *Point2D_1
     *Point2D_1\x = *Line2D\Position\x + i1 * *Line2D\Direction\x
     *Point2D_1\y = *Line2D\Position\y + i1 * *Line2D\Direction\y  
    EndIf
    If i2 >= 0 And i2 <= 1 
     If *Point2D_2
      *Point2D_2\x = *Line2D\Position\x + i2 * *Line2D\Direction\x
      *Point2D_2\y = *Line2D\Position\y + i2 * *Line2D\Direction\y  
     EndIf
     ProcedureReturn #True+#True
    Else
     ProcedureReturn #True
    EndIf
   Else
    If i2 >= 0 And i2 <= 1
     If *Point2D_1
      *Point2D_1\x = *Line2D\Position\x + i2 * *Line2D\Direction\x
      *Point2D_1\y = *Line2D\Position\y + i2 * *Line2D\Direction\y  
     EndIf
     ProcedureReturn #True
    Else
     ProcedureReturn #False
    EndIf
   EndIf
  EndIf
 EndProcedure
 
 
 
 ; Kopiert eine Linie 
 Macro CopyLine2D(Line2D, NewLine2D)
  CopyMemory(Line2D, NewLine2D, SizeOf(Line2D))
 EndMacro
 
 
 
 ; Vergleicht zwei Linien auf gleichheit
 Macro CompareLine2D(Line2D_1, Line2D_2)
  CompareMemory(Line2D_1, Line2D_2, SizeOf(Line2D))
 EndMacro
  
 ; Berechnet den Normalenvektor der Geraden
 Procedure Line2DNorm(*Line2D.Line2D, *NormVector2D.Vector2D)
  *NormVector2D\x = -*Line2D\Direction\y
  *NormVector2D\y = *Line2D\Direction\x
  NormVector2D(*NormVector2D)
 EndProcedure
  
  
  
 ; Gibt den Abstand zwischen einer Linie und einem Punkt zurck
 Procedure.f DistanceLinePoint2D(*Line2D.Line2D, *Vector2D.Vector2D)
  Protected DifferenceVector2D.Vector2D, NormVector2D.Vector2D, TempVector2D.Vector2D
  Vector2DSub(*Vector2D, *Line2D\Position, DifferenceVector2D)
  Line2DNorm(*Line2D, NormVector2D)
  Vector2DAdd(*Line2D\Position, *Line2D\Direction, TempVector2D)
  Protected Distance.f = Abs(Vector2DScalar(DifferenceVector2D, NormVector2D))
  Protected a.f = Distance2D(*Vector2D, *Line2D\Position)
  Protected b.f = Distance2D(*Vector2D, TempVector2D)
  If Vector2DLength(*Line2D\Direction)+0.001 >= Sqr(a*a-Distance*Distance)+Sqr(b*b-Distance*Distance)
   ProcedureReturn Distance
  ElseIf a < b 
   ProcedureReturn a
  Else
   ProcedureReturn b
  EndIf 
 EndProcedure 



 ; Stellt eine Linie dar
 Procedure DrawLine2D(*Line2D.Line2D, Color.q=-1, dx=0, dy=0)
  If *Line2D
   If Color > -1
    LineXY(*Line2D\Position\x+dx, *Line2D\Position\y+dy, *Line2D\Position\x+dx+*Line2D\Direction\x, *Line2D\Position\y+dy+*Line2D\Direction\y, Color)
   Else
    LineXY(*Line2D\Position\x+dx, *Line2D\Position\y+dy, *Line2D\Position\x+dx+*Line2D\Direction\x, *Line2D\Position\y+dy+*Line2D\Direction\y)
   EndIf
  EndIf
 EndProcedure
 
 
 
 ; Stellt einen Kreis dar
 Procedure DrawCircle2D(*Circle2D.Circle2D, Color=-1, dx=0, dy=0)
  If *Circle2D
   If Color > -1
    Circle(*Circle2D\Position\x+dx, *Circle2D\Position\y+dy, *Circle2D\Radius, Color)
   Else
    Circle(*Circle2D\Position\x+dx, *Circle2D\Position\y+dy, *Circle2D\Radius)
   EndIf
  EndIf
 EndProcedure 
 
; IDE Options = PureBasic 4.51 RC 1 (Windows - x86)
; CursorPosition = 269
; FirstLine = 247